'********************************************************************
'
' C-Control/BASIC       M20_TRX433.BAS (V2.20 v. 26.05.2005)
'
' Systemvoraussetzungen:
'
' - Application Board 2.0 mit C-Control I Unit M 2.0
' - Transceiver-Modul TRX 433
'
' Aufgaben:
'
' - Interface-Routinen zur Ansteuerung des Transceivers TRX 433
'   (CONRAD 190045) mit der C-Control I Unit M 2.0
' - Kompatibilitt zum C-Control II Modul "trx.c2" ab V2.20
'   (Siehe CC2Net.de!)
'
'********************************************************************

' Konstanten-Definitionen:

define MIN_POWER    3 'Fr TRX POWER_MIN
define TX_ADDR    255 'Senderadresse fr Control-TRX
define CONTROL_CODE 0 'Kontrollcode fr Control-TRX
define MODE_CTRL  255 'Kontrollmodus fr Control-TRX

' Variablen-Definitionen:

' TRX Benutzer Interface
define MODE     byte[1]
define SDATA1   byte[2]
define SDATA0   byte[3]
define COMMAND  byte[4]
define SADDRESS byte[5]

define DATA0    byte[2]
define DATA1    byte[3]
define DATA2    byte[4]
define DATA3    byte[5]
define DATA4    byte[6]
define DATA5    byte[7]
define DATA6    byte[8]
define ADDRESS  byte[9]

define TRX_BYTE byte[5] 'DATA3
define TRX_INT  word[3] 'DATA3/4

define RX_ERROR byte[10] 'Empfangsfehler [on/off]

' Benutzer Variablen
' (ab byte[11]!)


' Digitalport-Definitionen:
define DATA     port[1]  ' TRX 433 DATA  (blau)
define CLOCK    port[2]  ' TRX 433 CLOCK (gelb)
define ACKN     port[3]  ' TRX 433 ACKN  (grn)

' TRX 433 Treiber Aufrufe:
define SEND     &HFD03
define RECEIVE  &HFD7F

'********************************************************************

' Beginn Initialisierungen:

' Initialisierung TRX 433
gosub RESET_TRX 'Reset
'SDATA0=133:gosub SET_DEVICE_ADDRESS 'Z.B.: Gerteadresse=133
'gosub POWER_MAX:pause 2             'Z.B.: Max. Sendeleistung


'********************************************************************

' Beginn Hauptprogramm:


'********************************************************************

' TRX 433 Interface:

' 8 Byte RF-Frame senden
#SEND_RF
#SEND_RF8
MODE=0:sys SEND:return
' 8 Byte RF-Frame empfangen
#RECEIVE_RF
#RECEIVE_RF8
MODE=2:SADDRESS=0:COMMAND=13:sys SEND:pause 1
MODE=0:sys RECEIVE:return
' TRX Status lesen
#GET_STATUS
MODE=2:SADDRESS=0:COMMAND=0:sys SEND:sys RECEIVE
return 'Gerteadresse=SDATA1
' Kanal 0..4 (aus SDATA0) einstellen
#SET_CH
MODE=2:SADDRESS=0:COMMAND=7:sys SEND:return
' Kanal 0 einstellen
#CH0_ON
MODE=2:SADDRESS=0:COMMAND=7:SDATA0=0:sys SEND:return
' Kanal 1 einstellen
#CH1_ON
MODE=2:SADDRESS=0:COMMAND=7:SDATA0=1:sys SEND:return
' Kanal 2 einstellen
#CH2_ON 'Standard!
MODE=2:SADDRESS=0:COMMAND=7:SDATA0=2:sys SEND:return
' Kanal 3 einstellen
#CH3_ON
MODE=2:SADDRESS=0:COMMAND=7:SDATA0=3:sys SEND:return
' Kanal 4 einstellen
#CH4_ON
MODE=2:SADDRESS=0:COMMAND=7:SDATA0=4:sys SEND:return
' Sendeleistung 0..255 (aus SDATA0) einstellen
#SET_POWER
MODE=2:SADDRESS=0:COMMAND=6:sys SEND:return
' Sendeleistung -43 dBm einstellen
#POWER0
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=0:sys SEND:return
' Sendeleistung -20 dBm einstellen
#POWER1
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=1:sys SEND:return
' Sendeleistung -15 dBm einstellen
#POWER2
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=2:sys SEND:return
' Sendeleistung -10 dBm einstellen
#POWER3
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=4:sys SEND:return
' Sendeleistung -5 dBm einstellen
#POWER4
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=8:sys SEND:return
' Sendeleistung 0 dBm einstellen
#POWER5 'Standard!
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=15:sys SEND:return
' Sendeleistung +5 dBm einstellen
#POWER6
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=111:sys SEND:return
' Max. Sendeleistung +10 dBm einstellen
#POWER7
#POWER_MAX
MODE=2:SADDRESS=0:COMMAND=6:SDATA0=255:sys SEND:return
#POWER_MID
SDATA0=100:goto SET_POWER
#POWER_MIN
SDATA0=MIN_POWER:goto SET_POWER
' Reset TRX
#RESET_TRX
MODE=2:SADDRESS=0:COMMAND=8:sys SEND:pause 10
MODE=0:return
' CF-Framelnge 4 Byte whlen
#FRAME4
MODE=2:return
' RF-Framelnge 8 Byte einstellen
#FRAME8
MODE=2:SADDRESS=0:COMMAND=4:sys SEND
MODE=0:return
' RF-Framelnge 16 Byte einstellen
#FRAME16
MODE=2:SADDRESS=0:COMMAND=5:sys SEND
MODE=1:return
' Device Address 0..255 (aus SDATA0) einstellen
#SET_DEVICE_ADDRESS
MODE=2:SADDRESS=0:COMMAND=3:sys SEND
return
' Prambellnge (aus SDATA0) einstellen
#SET_PREAMBLE
MODE=2:SADDRESS=0:COMMAND=14:sys SEND
return

' RF-Buffer leeren
#FLUSH
if DATA=on then gosub RECEIVE_RF
return
' Warten (TRX_INT * 20ms) bis Daten gelesen werden knnen
#WAIT4READY
TRX_INT=TRX_INT+timer
#WAITING
if DATA=on then goto DATA_VALID
if TRX_INT<>timer then goto WAITING
TRX_INT=off:return 'Timeout
#DATA_VALID
TRX_INT=on:return  'Daten knnen gelesen werden
' Untere FSK Frequenz einstellen
#FSK_LO
MODE=2:SADDRESS=0:COMMAND=12:SDATA0=0:sys SEND:return
' Obere FSK Frequenz einstellen
#FSK_HI
MODE=2:SADDRESS=0:COMMAND=12:SDATA0=1:sys SEND:return
' Sender einschalten
#TX_ON
MODE=2:SADDRESS=0:COMMAND=9:sys SEND
return
' Empfnger einschalten
#RX_ON
MODE=2:SADDRESS=0:COMMAND=10:sys SEND
return
' Audio 1k senden
#TEST_MOD1K
#TX_AUDIO1K
MODE=2:SADDRESS=0:COMMAND=1:SDATA0=&H11:SDATA1=&H16:sys SEND
return
' Audio 2k senden
#TEST_MOD2K
#TX_AUDIO2K
MODE=2:SADDRESS=0:COMMAND=1:SDATA0=&H11:SDATA1=&H26:sys SEND
return

' Erweiterte 8-Byte-Sende-/Empfangsfunktionen
'  (Siehe C-Control II Modul "trx.c2" ab V2.2 [CC2Net.de]!)
' Kontrollframe senden/empfangen
'  Kontroll-Modus (1..19) in DATA0, Parameter in DATA1
'  Mgliche Werte fr DATA0:
'   Sendeleistung:           1   Direktkontrolle:   10
'   Kanal:                   2   Sendewiederholung: 11
'   Prambel:                3   Sendeanforderung:  12
'   Gerteadresse:           4   (Kontrollcode:     13)
'   Framelnge:              5
'   Informationsanforderung: 6
'  Nur fr Direktkontrolle (DATA0=10):
'   DATA4="COMMAND", DATA5="SDATA0", DATA6="SDATA1"
#SEND_CONTROL 'ADDRESS=Empfngeradresse
DATA3=CONTROL_CODE 'Kontrollcode
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=6 then goto Infoempfang
goto SEND_RF8
#Infoempfang
RX_ERROR=on
if DATA1=0 then goto Statusempfang
if DATA1=1 then goto Datumempfang
if DATA1=2 then goto Zeitempfang
goto SEND_RF8
#Statusempfang
gosub SEND_RF8
TRX_INT=14:gosub WAIT4READY 'max. 280 ms auf Daten warten
if TRX_INT=on then goto RECEIVE_STATUS
return
#Datumempfang
gosub SEND_RF8
TRX_INT=14:gosub WAIT4READY
if TRX_INT=on then goto RECEIVE_DATE
return
#Zeitempfang
gosub SEND_RF8
TRX_INT=14:gosub WAIT4READY
if TRX_INT=on then goto RECEIVE_TIME
return

#RECEIVE_CONTROL
gosub RECEIVE_RF
if MODE_CTRL=0 then goto Echo
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA2=RX_ERROR and DATA3=CONTROL_CODE then goto FrameOK
RX_ERROR=DATA0
#KontrollFehler
DATA0=RX_ERROR:RX_ERROR=on
return
#Echo
pause 9:gosub SEND_RF8:RX_ERROR=0
goto KontrollFehler
#FrameOK
RX_ERROR=DATA0
if MODE_CTRL=255 then goto ModusOK
if DATA0<>MODE_CTRL then goto KontrollFehler
#ModusOK
if TX_ADDR=255 or ADDRESS=TX_ADDR then goto Kontrolle
goto KontrollFehler
#Kontrolle
if DATA0=1 then goto Sendeleistung1
if DATA0=2 then goto Kanal2
if DATA0=3 then goto Praeambel3
if DATA0=4 then goto Adresse4
if DATA0=5 then goto Framelaenge5
if DATA0=6 then goto Info6
if DATA0=10 then goto Direktkontrolle10
if DATA0=11 or DATA0=12 then goto KontrolleOK
if DATA0=13 then goto Kontrollcode13
goto KontrollFehler
#Sendeleistung1
gosub SET_POWER '!!! SDATA0=DATA1 (byte[3]) !!!
goto KontrolleOK
#Kanal2
if DATA1>4 then goto KontrollFehler
gosub SET_CH
goto KontrolleOK
#Praeambel3
gosub SET_PREAMBLE
goto KontrolleOK
#Adresse4
gosub SET_DEVICE_ADDRESS
goto KontrolleOK
#Framelaenge5
if DATA1<>8 then goto KontrollFehler
gosub FRAME8:goto KontrolleOK
#Info6
if DATA1=0 then goto Status
if DATA1=1 then goto Datum
if DATA1=2 then goto Zeit
goto KontrollFehler
#Status
DATA3=15:DATA4=2:DATA5=10:DATA6=8 'Standardwerte!
pause 7:gosub SEND_STATUS
goto KontrolleOK
#Datum
pause 7:gosub SEND_DATE
goto KontrolleOK
#Zeit
pause 7:gosub SEND_TIME
goto KontrolleOK
#Direktkontrolle10
if DATA4>14 then goto KontrollFehler
SADDRESS=0:COMMAND=DATA4:SDATA0=DATA5:SDATA1=DATA6
if COMMAND=0 or COMMAND=5 or COMMAND=13 then goto KontrollFehler
MODE=2:sys SEND 'CF-Frame senden
if COMMAND=4 or COMMAND=8 then MODE=0
goto KontrolleOK
#Kontrollcode13
'CONTROL_CODE=DATA1 'CONTROL_CODE muss als Variable definiert sein!
goto KontrollFehler '<- ggf. entfernen!
#KontrolleOK
DATA0=RX_ERROR:RX_ERROR=off
return 'Modus 11/12: Sendewiederholung/-anforderung=DATA1,
       'Empfangsfehler=RX_ERROR

' Status senden/empfangen
'  DATA3=Sendeleistung, DATA4=Kanal, DATA5=Prambel,
'  DATA6=Framelnge
#SEND_STATUS 'ADDRESS=Empfngeradresse
DATA0=20 'MODE_STATUStrx
DATA1=0  'Frame Nr. 0
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
goto SEND_RF8

#RECEIVE_STATUS
gosub RECEIVE_RF
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=20 and DATA1=0 and DATA2=RX_ERROR then RX_ERROR=off else RX_ERROR=on
return 'Sendeleistung=DATA3, Kanal=DATA4, Prambel=DATA5,
       'Framelnge=DATA6, Empfangsfehler=RX_ERROR

' Byte (TRX_BYTE) senden/empfangen
#SEND_BYTE 'ADDRESS=Empfngeradresse
DATA0=21 'MODE_BYTEtrx
DATA1=0  'Frame Nr. 0
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
goto SEND_RF8

#RECEIVE_BYTE
gosub RECEIVE_RF
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=21 and DATA1=0 and DATA2=RX_ERROR then RX_ERROR=off else RX_ERROR=on
return 'Byte=TRX_BYTE, Empfangsfehler=RX_ERROR

' Integer (TRX_INT) senden/empfangen
#SEND_INT 'ADDRESS=Empfngeradresse
DATA0=22 'MODE_INTtrx
DATA1=DATA3:DATA3=DATA4:DATA4=DATA1 'LO/HI-Byte tauschen
DATA1=0  'Frame Nr. 0
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
goto SEND_RF8

#RECEIVE_INT
gosub RECEIVE_RF
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=22 and DATA1=0 and DATA2=RX_ERROR then RX_ERROR=off else RX_ERROR=on
DATA1=DATA3:DATA3=DATA4:DATA4=DATA1:DATA1=0 'LO/HI-Byte tauschen
return 'Integer=TRX_INT, Empfangsfehler=RX_ERROR

' Datum senden/empfangen
#SEND_DATE 'ADDRESS=Empfngeradresse
DATA0=26 'MODE_DATEtrx
DATA1=0  'Frame Nr. 0
DATA2=year 'Jahr funktioniert so bis 2099!
DATA3=208+DATA2:if DATA2>47 then DATA3=DATA2-48
DATA4=7:if DATA2>47 then DATA4=8
DATA5=month
DATA6=day
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
goto SEND_RF8

#RECEIVE_DATE
gosub RECEIVE_RF
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=26 and DATA1=0 and DATA2=RX_ERROR then goto DatumOK
RX_ERROR=on
return
#DatumOK
year=((DATA4*256+DATA3) MOD 100)
month=DATA5
day=DATA6
RX_ERROR=off
return 'Empfangsfehler=RX_ERROR

' Uhrzeit senden/empfangen
#SEND_TIME 'ADDRESS=Empfngeradresse
DATA0=27 'MODE_TIMEtrx
DATA1=0  'Frame Nr. 0
DATA3=hour
DATA4=minute
DATA5=second
DATA6=0  'CC1: 0, CC2: Sommerzeitflag (0=Normalzeit)
DATA2=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
goto SEND_RF8

#RECEIVE_TIME
gosub RECEIVE_RF
RX_ERROR=DATA0+DATA1+DATA3+DATA4+DATA5+DATA6 'Checksumme
if DATA0=27 and DATA1=0 and DATA2=RX_ERROR then goto ZeitOK
RX_ERROR=on
return
#ZeitOK
hour=DATA3
minute=DATA4
second=DATA5
RX_ERROR=off
return 'Empfangsfehler=RX_ERROR

'********************************************************************

' Gertetreiber fr TRX 433 laden:
syscode "M20_TRX433_PAGE0.s19"
